from django.db import models

__all__ = [
    "CreationDateMixin"
]


class CreationDateMixin(models.Model):
    """Add a `created_at` field which is automatically set on model creation."""

    class Meta:
        """Meta class."""

        abstract = True

    created_at = models.DateTimeField(
        auto_now_add=True
    )



from model_mixin_test import ModelMixinTestCase
from django_model_mixins.mixins import TokenMixin


class TokenMixinTest(ModelMixinTestCase):
    """Test cases for `TokenMixin`."""

    mixin = TokenMixin
    model: TokenMixin

    def test_creates_id(self):
        """Test if a random `token` is generated."""
        instance = self.model.objects.create()
        self.assertNotEqual("", instance.token)

    def test_id_length_is_derived_from_model(self):
        """Test if length is derived from the model."""
        old_value = self.model.TOKEN_LENGTH
        length = 10
        self.model.TOKEN_LENGTH = length

        instance = self.model.objects.create()
        self.assertEqual(length, len(instance.token))

        self.model.TOKEN_LENGTH = old_value

    def test_doesnt_create_id_if_is_false(self):
        """Test if a `token` is not generated when `CREATE_ON_CREATION = False`."""
        old_value = self.model.CREATE_ON_CREATION

        self.model.CREATE_ON_CREATION = False
        instance = self.model.objects.create()
        self.assertEqual("", instance.token)

        self.model.CREATE_ON_CREATION = old_value

    def test_token_can_be_recreated(self):
        """Test if a `token` can be recreated."""
        instance = self.model.objects.create()
        old_token = instance.token

        instance.recreate_token()
        instance.refresh_from_db()

        self.assertNotEqual(old_token, instance.token)



from datetime import datetime, timedelta, timezone

from django.db import models

from .token import TokenMixin

__all__ = [
    "TokenGeneratedAtMixin"
]


class TokenGeneratedAtMixin(TokenMixin):
    """Inherits `TokenMixin` and add a `token_generated` field.

    This class expects a field called `TOKEN_VALID_DURATION_IN_SECONDS` which is an int
    indicating how long a `token` is valid (duration in seconds).

    `token_generated` saves when the given `token` has been generated.
    """

    class Meta:
        """Meta class."""

        abstract = True

    # Duration in seconds
    TOKEN_VALID_DURATION_IN_SECONDS: int

    token_generated = models.DateTimeField(
        blank=True,
        null=True
    )

    @property
    def expire_date(self) -> datetime:
        """Return the datetime this token will expire."""
        return self.token_generated + timedelta(seconds=self.TOKEN_VALID_DURATION_IN_SECONDS)

    @property
    def is_expired(self) -> bool:
        """Return whether this token has expired."""
        return self.expire_date <= timezone.now()

    def _create_token(self):
        """Generate a new token and set it."""
        super()._create_token()
        self.token_generated = timezone.now()



import string

__all__ = [
    "DEFAULT_RANDOM_ID_MAX_LENGTH", "DEFAULT_RANDOM_ID_CHARS", "DEFAULT_TOKEN_MAX_LENGTH",
    "DEFAULT_TOKEN_CHARS", "TOKEN_CREATE_ON_CREATION"
]

DEFAULT_RANDOM_ID_MAX_LENGTH = 1023
DEFAULT_RANDOM_ID_CHARS = string.ascii_letters + string.digits
DEFAULT_TOKEN_MAX_LENGTH = 2047
DEFAULT_TOKEN_CHARS = string.ascii_letters + string.digits
TOKEN_CREATE_ON_CREATION = True



import secrets

from django.conf import settings
from django.db import models

from .. import constants

__all__ = [
    "RandomIDMixin"
]

ID_MAX_LENGTH = getattr(settings, "RANDOM_ID_MAX_LENGTH", constants.DEFAULT_RANDOM_ID_MAX_LENGTH)
ID_CHARS = getattr(settings, "RANDOM_ID_CHARS", constants.DEFAULT_RANDOM_ID_CHARS)


class RandomIDMixin(models.Model):
    """Add a `id` field which is randomly set and unchangeable."""

    class Meta:
        """Meta class."""

        abstract = True

    ID_LENGTH = ID_MAX_LENGTH
    ID_CHARS = ID_CHARS

    id = models.CharField(
        max_length=ID_MAX_LENGTH,
        primary_key=True,
        unique=True,
        editable=False
    )

    @classmethod
    def _generate_id(cls) -> str:
        """Generate a cryptographically randomly and unique id.

        You can override this function if you want to change the generation.
        This function doesn't set the `id` - it only generates and returns it.
        """
        assert cls.ID_LENGTH <= ID_MAX_LENGTH, f"`ID_LENGTH` can be at most `{ID_MAX_LENGTH}`."

        # Create an id until it's available

        # Because of the fact, that it's very unlikely that an id will be taken already,
        # we will do a database lookup instead of saving the ids in a set. This reduces the
        # amount of data that must be stored in the memory.

        while True:
            random_id = "".join(
                secrets.choice(cls.ID_CHARS)
                for _ in range(cls.ID_LENGTH)
            )

            # Check availability
            id_taken = cls.objects.only("id").filter(id=random_id).exists()
            if not id_taken:
                return random_id

    def save(self, *args, **kwargs):
        """Automatically set `id`."""
        if not self.id:
            self.id = self._generate_id()

        return super().save(*args, **kwargs)



import secrets

from django.conf import settings
from django.db import models

from .. import constants

__all__ = [
    "TokenMixin"
]

TOKEN_MAX_LENGTH = getattr(settings, "TOKEN_MAX_LENGTH", constants.DEFAULT_TOKEN_MAX_LENGTH)
TOKEN_CHARS = getattr(settings, "TOKEN_CHARS", constants.DEFAULT_TOKEN_CHARS)
CREATE_ON_CREATION = getattr(
    settings, "TOKEN_CREATE_ON_CREATION", constants.TOKEN_CREATE_ON_CREATION
)


class TokenMixin(models.Model):
    """Create a `token` field with the ability to change it."""

    class Meta:
        """Meta class."""

        abstract = True

    TOKEN_LENGTH = TOKEN_MAX_LENGTH
    TOKEN_CHARS = TOKEN_CHARS
    CREATE_ON_CREATION = CREATE_ON_CREATION

    token = models.CharField(
        max_length=TOKEN_MAX_LENGTH
    )

    @classmethod
    def _generate_token(cls) -> str:
        """Generate a token and return it.

        This function just generates a token.
        """
        assert cls.TOKEN_LENGTH <= TOKEN_MAX_LENGTH, \
            f"`TOKEN_LENGTH` can be at most `{TOKEN_MAX_LENGTH}`"

        return "".join(
            secrets.choice(cls.TOKEN_CHARS)
            for _ in range(cls.TOKEN_LENGTH)
        )

    def _create_token(self):
        """Generate a new token and set it."""
        self.token = self._generate_token()

    def recreate_token(self):
        """Recreates the token and saves it."""
        self._create_token()
        self.save()

    def save(self, *args, **kwargs):
        """Create a token if necessary and allowed."""
        if self.token == "" and self.CREATE_ON_CREATION:
            self._create_token()

        return super().save(*args, **kwargs)



from django.db import models

__all__ = [
    "EditDateMixin"
]


class EditDateMixin(models.Model):
    """Mixin to create a `edited_at` field which is automatically set on model update."""

    class Meta:
        """Meta class."""

        abstract = True

    edited_at = models.DateTimeField(
        auto_now=True
    )



from django.apps import AppConfig


class DjangoModelMixinsConfig(AppConfig):
    """Configuration for `django-model-mixins`."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "django_model_mixins"



from model_mixin_test import ModelMixinTestCase
from django_model_mixins.mixins import RandomIDMixin


class RandomIDMixinTest(ModelMixinTestCase):
    """Test cases for `RandomIDMixin`."""

    mixin = RandomIDMixin
    model: RandomIDMixin

    def test_creates_id(self):
        """Test if a random `id` is generated."""
        instance = self.model.objects.create()
        self.assertIsNotNone(instance.id)
        self.assertNotEqual("", instance.id)

    def test_id_length_is_derived_from_model(self):
        """Test if length is derived from the model."""
        old_value = self.model.ID_LENGTH
        length = 10
        self.model.ID_LENGTH = length
        instance = self.model.objects.create()
        self.assertEqual(length, len(instance.id))
        self.model.ID_LENGTH = old_value



from pathlib import Path

import pypandoc
from setuptools import setup, find_packages

current_version = "0.0.3"
download_url = f"https://github.com/Myzel394/django-model-mixins/archieve/v_" \
               f"{current_version}.tar.gz"

dev_requires = [
    "flake8",
    "flake8-docstrings",
    "flake8-dunder-all",
    "flake8-quotes"
]


def get_long_description(name: str = "README.md"):
    try:
        return pypandoc.convert("README.md", "rst")
    except (IOError, ImportError):
        description=open('README.md').read()


def get_install_requires():
    return Path.cwd().joinpath("requirements.txt").read_text().splitlines()


setup(
    name="django-model-mixins",
    version=current_version,
    author="Myzel394 Xyllian",
    author_email="myzel394.xyllian@gmail.com",
    url="https://github.com/Myzel394/django-model-mixins",
    description="Mixins for Django models.",
    long_description=get_long_description(),
    platforms=['OS Independent'],
    license="MIT",
    download_url=download_url,
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Framework :: Django",
        "Intended Audience :: Developers",
    ],
    keywords=["django", "django-utils", "django-models"],
    install_requires=get_install_requires(),
    extra_require={
        "dev": dev_requires
    }
)
